#ifndef MUSTUXCOMPONENT_HH
#define MUSTUXCOMPONENT_HH

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <string>

class MustuxComponent;
class MustuxWindow;

/** This is a base class used to "hold" other components.
 *  A MustuxComponet must have a parent where it is allocated in,
 *  this parental component can be either a MustuxWindow or another MustuxComponent
 */
class MustuxComponent
    {
    public :
        /** Constructs a MustuxComponent inside a MustuxWindow        
         *  @param aRootWindow : The window that will contain this component
         */
        MustuxComponent(MustuxWindow *aRootWindow);

        /** Constructs a MustuxComponent inside another MustuxComponent        
         *  @param aRootComponent : The component that will contain this component
         */
        MustuxComponent(MustuxComponent *aParentComponent);

        /** Constructs a MustuxComponet that is linked to the parent through its box
         *
         */
        MustuxComponent(GtkWidget* aParentBox);

        virtual ~MustuxComponent();

        /** Build graphically the component with the following standard properties :
         *  a) distribute sub-component (if any) in a VERTICAL direction
         *  b) Has non-homogeneous distribution (sub components can have different sizes)
         *  c) Resizeable 
         *  d) Fillable - it will allocate all space avaiable for every side (up, down , left and right)
         */
        int build();


        /** Build graphically the component with the following properties :
         *  a) sub-components distribution given by "direction" - can be
         *     MustuxComponent::VERTICAL or MustuxComponent::HORIZONTAL
         *  b) Has non-homogeneous distribution (sub components can have different sizes)
         *  c) Resizeable 
         *  d) Fillable - it will allocate all space avaiable for every side (up, down , left and right)
         */
        int build(int direction);


        /** Build graphically the component with the following properties :
         *  a) sub-components distribution given by "direction" - can be
         *     MustuxComponent::VERTICAL or MustuxComponent::HORIZONTAL
         *  b) Has homogeneous distribution if "isHomogeneous" is true
         *  c) Resizeable 
         *  d) Fillable - it will allocate all space avaiable for every side (up, down , left and right)
         */
        int build(int direction, bool isHomogeneous);


        /** Build graphically the component with the following properties :
         *  a) sub-components distribution given by "direction" - can be
         *     MustuxComponent::VERTICAL or MustuxComponent::HORIZONTAL
         *  b) Has homogeneous distribution if "isHomogeneous" is true
         *  c) will be Resizeable if isResizeable is true
         *  d) Fillable - it will allocate all space avaiable for every side (up, down , left and right)
         */
        int build(int direction, bool isHomogeneous, bool isResizeable);

        /** Build graphically the component with the following properties :
         *  a) sub-components distribution given by "direction" - can be
         *     MustuxComponent::VERTICAL or MustuxComponent::HORIZONTAL
         *  b) Has homogeneous distribution if "isHomogeneous" is true
         *  c) will be Resizeable if isResizeable is true
         *  d) will be Fillable if isFillable is true
         */
        int build(int direction, bool isHomogeneous, bool isResizeable, bool isFillable);

        /** Initialize the component and internal properties
        */
        int init();

        /** Prepare the component to be unbuilt. 
         *  Normally called automatically by event handlers.
         */
        int finalize();


        /** Unbuild graphically the component and de-allocate memory used. 
         *  Normally called automatically by event handlers.
         */
        int unbuild();


        /** Set the direction to distribute sub-components
         */
        void set_fill_direction(int newFillDirection);
        
        /** Add a separator (vertical or horizontal, according to distribution direction)
         *
         */
        void add_separator();

        /** Add a test widget (for debugging purposes)
         *
         */
        void add_test_widget()         ;

        /** Returns the parent component containing this one.
         */
        MustuxComponent* get_parent();

        /** Returns the GTK box that is holding internal widgets. Usefull for
         *  adding new GTK widgets into it.
         */
        GtkWidget* get_box();

        static const int VERTICAL=1;
        static const int HORIZONTAL=2;
        static const bool RESIZEABLE=true;
        static const bool NOT_RESIZEABLE=false;
        static const bool HOMOGENEOUS=true;
        static const bool NOT_HOMOGENEOUS=false;
        static const bool FILLABLE=true;
        static const bool NOT_FILLABLE=false;

    protected :
        MustuxComponent *parent;
        MustuxWindow *rootWindow;
        GtkWidget *box;
        GtkWidget *parentBox;
        int fillDirection;
    };


#include "MustuxWindow.hh"


#endif

